@@ -1,5 +1,3 @@
-{-# OPTIONS_GHC -fglasgow-exts #-}
-
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Arrow.Operations
@@ -33,7 +31,6 @@ module Control.Arrow.Operations (
 	) where
 
 import Control.Arrow
-import Control.Category ((>>>))
 import Data.Monoid
 
 -- $conventions
