@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-}
+{-# LANGUAGE FlexibleInstances #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -15,7 +15,7 @@
 -- /TODO:/ the operations here are inconsistent with other arrow transformers.
 
 module Control.Arrow.Transformer.Error(
-		ErrorArrow,
+		ErrorArrow(ErrorArrow),
 		runError,
 		ArrowAddError(..),
 	) where
