@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-}
+{-# LANGUAGE FlexibleInstances #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -13,7 +13,7 @@
 -- Arrow transformer that adds a read-only state (i.e. an environment).
 
 module Control.Arrow.Transformer.Reader(
-		ReaderArrow,
+		ReaderArrow(ReaderArrow),
 		runReader,
 		ArrowAddReader(..),
 	) where
