@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-}
+{-# LANGUAGE FlexibleInstances #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -13,7 +13,7 @@
 -- Arrow transformer that adds accumulation of output.
 
 module Control.Arrow.Transformer.Writer(
-		WriterArrow,
+		WriterArrow(WriterArrow),
 		runWriter,
 		ArrowAddWriter(..),
 	) where
